import sys

from pretty_plz.env import Env
from pretty_plz.brain import pretty_plzBrain
from pretty_plz.return_codes import ReturnCodes
from pretty_plz.exceptions import CommandNotFound


def main():
  if Env.scripts_path() == None:
    print("Error: set $PLZ_SCRIPTS_PATH!")
    sys.exit(ReturnCodes.NO_PLZ_DIR)

  from argparse import ArgumentParser, REMAINDER
  parser = ArgumentParser()
  parser.add_argument("command")
  parser.add_argument("cli", nargs=REMAINDER)
  args = parser.parse_args()

  try:
    brain = pretty_plzBrain(args.command)
    resolved_cmd = brain.find_command()
    import subprocess
    proc = subprocess.run([*resolved_cmd, *args.cli])
    sys.exit(proc.returncode)
  except CommandNotFound as exc:
    print(f"Error: {exc}")
    sys.exit(ReturnCodes.NOT_FOUND)
  except PermissionError:
    print("Error: command found, but it isn't executable!")
    sys.exit(ReturnCodes.NOT_EXECUTABLE)



from os import getenv
from functools import wraps


def _env(var_name, default=None, var_type=str):
  def real_decorator(func):
    @wraps(func)
    def wrapped(*args, **kwargs):
      func()
      value = getenv(var_name, default=default)
      if value == default:
        return default
      return var_type(value)
    return wrapped
  return real_decorator


class Env:
  @_env("PLZ_SCRIPTS_PATH")
  def scripts_path():
    pass



import sys

from pathlib import Path

from pretty_plz.env import Env
from pretty_plz.exceptions import CommandNotFound


runnable_extensions = (
  ".exe", ".bat", ".cmd", ".sh", ".zsh", ".bash",
)


interpreter_extensions = {
  ".py": sys.executable,
  ".js": "node",
  ".ts": "ts-node",
  ".pl": "perl",
}


class pretty_plzBrain:
  def __init__(self, command):
    self.command = Path(command)
    self.spath = Path(Env.scripts_path())

  def runnable_candidates(self):
    yield self.spath / self.command
    for ext in runnable_extensions:
      yield self.spath / (str(self.command) + ext)

  def interp_candidates(self):
    for ext, interp in interpreter_extensions.items():
      yield (self.spath / (str(self.command) + ext), interp, )

  def find_command(self):
    for item in self.runnable_candidates():
      if item.exists():
        return (str(item),)
    for item, interp in self.interp_candidates():
      if item.exists():
        return (interp, str(item), )
    raise CommandNotFound(f"Could not find command `{str(self.command)}`")



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pretty_plz']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['plz = pretty_plz.main:main']}

setup_kwargs = {
    'name': 'pretty-plz',
    'version': '0.1.3',
    'description': 'pretty_plz is a tool that turns utility scripts into runnable commands',
    'long_description': None,
    'author': 'Tamás Szelei',
    'author_email': 'szelei.t@gmail.com',
    'url': 'https://github.com/sztomi/pretty_plz/',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
